/*
 * #%L
 * JRst :: Api
 * 
 * $Id: JRST.java 628 2011-11-29 17:38:54Z sletellier $
 * $HeadURL: http://svn.nuiton.org/svn/jrst/tags/jrst-1.5/jrst/src/main/java/org/nuiton/jrst/JRST.java $
 * %%
 * Copyright (C) 2004 - 2011 CodeLutin, Chatellier Eric
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation, either version 3 of the 
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Lesser Public License for more details.
 * 
 * You should have received a copy of the GNU General Lesser Public 
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/lgpl-3.0.html>.
 * #L%
 */
package org.nuiton.jrst;

import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.io.*;
import java.net.URL;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.sax.SAXResult;
import javax.xml.transform.stream.StreamSource;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.fop.apps.FOUserAgent;
import org.apache.fop.apps.Fop;
import org.apache.fop.apps.FopFactory;
import org.apache.fop.apps.MimeConstants;
import org.dom4j.Document;
import org.nuiton.i18n.I18n;
import org.nuiton.i18n.init.ClassPathI18nInitializer;
import org.nuiton.jrst.convertisor.DocUtils2RST;
import org.nuiton.jrst.convertisor.DocUtilsVisitor;
import org.nuiton.util.ApplicationConfig;
import org.nuiton.util.FileUtil;
import org.nuiton.util.Resource;
import org.nuiton.util.StringUtil;

/**
 * FIXME: 'JRST --help' doesn't work, but 'JRST --help toto' work :( FIXME:
 * 'JRST -c' doesn't work, but 'JRST -c toto'
 * 
 * Created: 3 nov. 06 20:56:00
 *
 * @author poussin
 * @version $Revision: 628 $
 *
 * Last update: $Date: 2011-11-29 18:38:54 +0100 (mar, 29 nov 2011) $
 * by : $Author: sletellier $
 */
public class JRST {

    public static final String UTF_8 = "UTF-8";

    public enum Overwrite {
        NEVER, IFNEWER, ALLTIME
    }

    /** to use log facility, just put in your code: log.info("..."); */
    protected static Log log = LogFactory.getLog(JRST.class);

    /** XSL Stylesheet to transform RST into HTML. */
    protected static final String rst2xhtml = "/xsl/rst2xhtml.xsl";
    /** XSL Stylesheet to transform RST into HTML (but only inner body fragment). */
    protected static final String rst2xhtmlInnerBody = "/xsl/rst2xhtmlInnerBody.xsl";
    /** XSL Stylesheet to transform RST into Xdoc. */
    protected static final String rst2xdoc = "/xsl/rst2xdoc.xsl";
    /** XSL Stylesheet to transform RST into Docbook. */
    protected static final String rst2docbook = "/xsl/dn2dbk.xsl";

    /** XSL Stylesheet to transform Docbook into xHTML. */
    protected static final String docbook2xhtml = "/docbook/xhtml/docbook.xsl";
    /** XSL Stylesheet to transform Docbook into javahelp. */
    protected static final String docbook2javahelp = "/docbook/javahelp/javahelp.xsl";
    /** XSL Stylesheet to transform Docbook into htmlhelp. */
    protected static final String docbook2htmlhelp = "/docbook/htmlhelp/htmlhelp.xsl";
    
//    /** XSL Stylesheet to transform xml into rst. */
//    protected static final String rst2rst = "JRSTWriter";

    /** XSL Stylesheet to transform Docbook into PDF. */
    protected static final String docbook2fo = "/docbook/fo/docbook.xsl";

    public static final String PATTERN_TYPE = "xml|xhtml|docbook|html|htmlInnerBody|xdoc|rst|fo|pdf|odt|rtf";

    /** HTML output format type */
    public static final String TYPE_HTML = "html";

    /** HTML output format type */
    public static final String TYPE_HTML_INNER_BODY = "htmlInnerBody";

    /** XDOC output format type */
    public static final String TYPE_XDOC = "xdoc";

    /** DOCBOOK output format type */
    public static final String TYPE_DOCBOOK = "docbook";

    /** XHTML output format type */
    public static final String TYPE_XHTML = "xhtml";

    /** JAVA HELP output format type */
    public static final String TYPE_JAVAHELP = "javahelp";

    /** HTML HELP output format type */
    public static final String TYPE_HTMLHELP = "htmlhelp";

    /** RST output format type */
    public static final String TYPE_RST = "rst";

    /** ODT output format type */
    public static final String TYPE_ODT = "odt";

    /** FO output format type */
    public static final String TYPE_FO = "fo";

    /** PDF output format type */
    public static final String TYPE_PDF = "pdf";

    /**
     * key, Out type; value: chain of XSL file to provide wanted file for output
     */
    protected static Map<String, String> stylesheets;
    /**
     * Mime type associated with type
     */
    protected static Map<String, String> mimeType;

    static {
        stylesheets = new HashMap<String, String>();
        stylesheets.put(TYPE_HTML, rst2xhtml);
        stylesheets.put(TYPE_HTML_INNER_BODY, rst2xhtmlInnerBody);
        stylesheets.put(TYPE_XDOC, rst2xdoc);
        stylesheets.put(TYPE_DOCBOOK, rst2docbook);
        stylesheets.put(TYPE_XHTML, rst2docbook + "," + docbook2xhtml);
        stylesheets.put(TYPE_JAVAHELP, rst2docbook + "," + docbook2javahelp);
        stylesheets.put(TYPE_HTMLHELP, rst2docbook + "," + docbook2htmlhelp);
        stylesheets.put(TYPE_RST, "");
        //stylesheets.put(TYPE_ODT, rst2docbook + "," + docbook2odf);
        stylesheets.put(TYPE_FO, rst2docbook + "," + docbook2fo);
        stylesheets.put(TYPE_PDF, rst2docbook + "," + docbook2fo);

        mimeType = new HashMap<String, String>();
        mimeType.put(TYPE_HTML, "text/html");
        mimeType.put(TYPE_HTML_INNER_BODY, "text/html");
        mimeType.put(TYPE_XDOC, "text/xml");
        mimeType.put(TYPE_DOCBOOK, "text/xml");
        mimeType.put(TYPE_XHTML, "text/html");
        mimeType.put(TYPE_JAVAHELP, "text/plain");
        mimeType.put(TYPE_HTMLHELP, "text/html");
        mimeType.put(TYPE_RST, "text/plain");
        mimeType.put(TYPE_ODT, "application/vnd.oasis.opendocument.text");
        mimeType.put(TYPE_FO, "text/xml");
        mimeType.put(TYPE_PDF, "application/pdf");

    }

    static public String getMimeType(String generationType) {
        String result = mimeType.get(generationType);
        if (result == null) {
            result = "application/octet-stream";
        }
        return result;
    }

    /**
     * Main method.
     * 
     * @param args main args
     * @throws Exception 
     */
    public static void main(String[] args) throws Exception {
        
        I18n.init(new ClassPathI18nInitializer(), Locale.UK);

        if (args.length == 0) {
            args = askOption();
        }
        if (args == null) {
            System.exit(0);
        }

        ApplicationConfig config = JRSTConfig.getConfig(args);
        config.doAction(0);

        // parse options
        String xslList = JRSTConfigOption.XSL_FILE.getOption(config);
        if (xslList == null) {
            xslList = JRSTConfigOption.OUT_TYPE.getOption(config);
        }
        List<String> unparsed = config.getUnparsed();
        if (unparsed.isEmpty()) {
            JRSTConfig.help();
        }
        File inputFile = new File(config.getUnparsed().get(0));
        File ouputFile = JRSTConfigOption.OUT_FILE.getOptionAsFile(config);
        Overwrite overwrite = Overwrite.NEVER;
        if (JRSTConfigOption.FORCE.getOptionAsBoolean(config)) {
            overwrite = Overwrite.ALLTIME;
        }
        
        generate(xslList, inputFile, ouputFile, overwrite);
    }

    private static String[] askOption() throws SecurityException,
            NoSuchMethodException, IOException {
        String[] result = null;
        try {
            GraphicsEnvironment ge = GraphicsEnvironment
                    .getLocalGraphicsEnvironment();
            GraphicsDevice[] gs = ge.getScreenDevices();
            if (!(gs == null)) {
                if (gs.length > 0) {
                    result = askOptionGraph();
                }
            }
        } catch (java.awt.HeadlessException e) {
            result = null;
        }

        return result;

    }

    /**
     * Interface graphique.
     * 
     * @return
     * @throws SecurityException
     * @throws NoSuchMethodException
     */
    protected static String[] askOptionGraph() throws SecurityException,
            NoSuchMethodException {

        JRSTInterface graph = new JRSTInterface(PATTERN_TYPE);

        return graph.getCmd();
    }

    public static void generate(String xslListOrOutType, File fileIn,
            File fileOut, Overwrite overwrite) throws Exception {

        generate(xslListOrOutType, fileIn, UTF_8, fileOut, UTF_8, overwrite);
    }

    public static void generate(String xslListOrOutType, File fileIn, String inputEncoding,
            File fileOut, String outputEncoding, Overwrite overwrite) throws Exception {

        if (fileOut != null
                && fileOut.exists()
                && (overwrite == Overwrite.NEVER || (overwrite == Overwrite.IFNEWER && FileUtil
                        .isNewer(fileIn, fileOut)))) {
            // System.err.println("Don't generate file " + fileOut +
            // ", because already exists");
            log.info("Don't generate file " + fileOut
                    + ", because already exists");
        } else {

            // In
            URL url = fileIn.toURI().toURL();
            Reader reader = new InputStreamReader(url.openStream(), inputEncoding);

            // Out
            FileOutputStream outputStream = new FileOutputStream(fileOut);
            OutputStreamWriter writer = new OutputStreamWriter(outputStream, outputEncoding);

            // Generate
            String result = generateString(xslListOrOutType, reader);

            fileOut.getAbsoluteFile().getParentFile().mkdirs();

            // generation PDF
            if (xslListOrOutType.equals("pdf")) {
                FopFactory fopFactory = FopFactory.newInstance();
                // OutputStream outPDF = new BufferedOutputStream(new
                // FileOutputStream(new File("C:/Temp/myfile.pdf")));

                OutputStream outPDF = new BufferedOutputStream(new FileOutputStream(
                        fileOut));

                FOUserAgent userAgent = fopFactory.newFOUserAgent();

                // Step 3: Construct fop with desired output format
                Fop fop = fopFactory.newFop(MimeConstants.MIME_PDF,
                        userAgent, outPDF);

                // Step 4: Setup JAXP using identity transformer
                TransformerFactory factory = TransformerFactory
                        .newInstance();
                Transformer transformer = factory.newTransformer(); // identity
                // transformer

                // Step 5: Setup input and output for XSLT transformation
                // Setup input stream
                Source src = new StreamSource(new StringReader(result));

                // Resulting SAX events (the generated FO) must be piped
                // through to FOP
                Result res = new SAXResult(fop.getDefaultHandler());

                // Step 6: Start XSLT transformation and FOP processing
                transformer.transform(src, res);

                outPDF.close();
            } else {
                // write generated document
                writer.write(result);

                writer.close();

            }
        }
    }

    /**
     * 
     * @param xslListOrOutType
     * @param in
     * @throws Exception
     */
    public static String generate(String xslListOrOutType, String in) throws Exception {
        return generateString(xslListOrOutType, new StringReader(in));
    }

    /**
     * 
     * @param xslListOrOutType
     * @param in
     * @throws Exception
     */
   public static Writer generate(String xslListOrOutType, Reader in) throws Exception {
       StringWriter writer = new StringWriter();
       writer.write(generateString(xslListOrOutType,in));
       return writer;
   }
   
   
   /**
   *
   * @param xslListOrOutType
   * @param in
   * @throws Exception
   */
  public static StringBuffer generate(String xslListOrOutType, StringBuffer in) throws Exception {
      return new StringBuffer(generate(xslListOrOutType, in.toString()));
  }
   
   
    /**
     *
     * @param xslListOrOutType
     * @param in
     * @throws Exception
     */
    public static String generateString(String xslListOrOutType, Reader in) throws Exception {

        Document doc = generateXML(in);

        // Sortie vers rst
        if (xslListOrOutType.equals(TYPE_RST)){
            return generateRST(doc);
        } else if (!StringUtils.isEmpty(xslListOrOutType)) {
            doc = applyXsls(doc, xslListOrOutType);
        }
        return doc.asXML();
    }

    private static Document applyXsls(Document doc, String xslListOrOutType) throws IOException, TransformerException {

        // search xsl file list to apply
        String xslList = stylesheets.get(xslListOrOutType);
        if (xslListOrOutType == null) {
            xslList = xslListOrOutType;
        }

        // apply xsl on rst xml document
        JRSTGenerator gen = new JRSTGenerator();
        String[] xsls = StringUtil.split(xslList, ",");
        for (String xsl : xsls) {
            URL stylesheet = null;
            File file = new File(xsl);
            if (file.exists()) {
                stylesheet = file.toURI().toURL();
            } else {
                //stylesheet = JRST.class.getResource(xsl);
                stylesheet = Resource.getURL(xsl);
            }
            if (stylesheet == null) {
                throw new FileNotFoundException("Can't find stylesheet: "
                        + xsl);
            }

            // add entity resolver
            gen.setUriResolver(new JRSTResourceResolver(xsl));

            // do transformation
            doc = gen.transform(doc, stylesheet);
        }
        return doc;
    }

    public static Document generateXML(Reader in) throws Exception {
        // parse rst file
        JRSTReader jrst = new JRSTReader();
        return jrst.read(in);
    }

    public static String generateRST(Document doc) throws IOException {
        // Creation d'un visitor qui convertie de l'xml vers le rst
        DocUtilsVisitor visitor = new DocUtils2RST();

        // Atacher le visitor au document
        // il va parcourir tout les elements et reconstruire du rst
        doc.accept(visitor);

        // Recuperation du resultat
        String result = visitor.getResult();
        // nettoyage du visiteur
        visitor.clear();

        return result;
    }
}
